/*
 * StrideCPUSpec.java
 * Resource specification for a stride scheduled CPU
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Lesser 
 * General Public License.
 * 
 * See the file "license-lesser.terms" for information on 
 * usage and redistribution of this file.
 */

package edu.utah.janosvm.resources;

/**
 * Specification for a stride scheduled CPU.
 */
public final class StrideCPUSpec
    extends CPUSpec
{
    /**
     * The Kernel's ticket count, mainly for reference when determining your
     * own ticket counts.
     */
    public static final int KERNEL_TICKETS = 10000;

    /**
     * The default ticket count.
     */
    public static final int DEFAULT_TICKETS = 100;

    /**
     * Cached reference to this class.
     */
    private static final Class THIS_CLASS = StrideCPUSpec.class;
    
    /**
     * Constructs the native part of the class.
     */
    private native void nativeConstructor(int tickets);

    /**
     * Construct a CPU resource specification that requires the number of
     * shares for the new team.
     *
     * @param limit The CPU usage limit.
     * @param tickets The number of tickets to give the team.
     */
    public StrideCPUSpec(long limit, int tickets)
    {
	super(limit);
	
	if( tickets <= 0 )
	{
	    throw new IllegalArgumentException("Negative ticket count: "
					       + tickets);
	}
	this.nativeConstructor(tickets);
    }

    /**
     * Construct a CPU resource specification that requires the number of
     * shares for the new team.
     *
     * @param tickets The number of tickets to give the team.
     */
    public StrideCPUSpec(int tickets)
    {
	this(INFINITE_LIMIT, tickets);
    }

    /**
     * @return The ticket count in this specification.
     */
    public native int getTickets();

    public String toString()
    {
	return "StrideCPUSpec["
	    + super.toString()
	    + "; tickets="
	    + this.getTickets()
	    + "]";
    }

    /**
     * Finalizer for any native data.
     */
    private native void nativeFinalizer();

    public boolean equals(Object obj)
    {
	boolean retval = false;

	if( obj instanceof StrideCPUSpec )
	{
	    StrideCPUSpec cs;

	    cs = (StrideCPUSpec)obj;
	    retval = (this.getTickets() == cs.getTickets());
	}
	return retval;
    }

    protected void finalize()
    {
	this.nativeFinalizer();
    }

    public native Object clone();

    /**
     * @return Construct a CPU resource specification that contains the default
     * ticket count.
     */
    public static StrideCPUSpec defaultSpec()
    {
	return new StrideCPUSpec(DEFAULT_TICKETS);
    }
    
    /**
     * @return The StrideCPUSpec for the current team.
     */
    public static StrideCPUSpec current()
    {
	return (StrideCPUSpec)CPUSpec.current(THIS_CLASS);
    }
}
