/*
 * TeamEjectionException.java
 * Exception thrown when a thread is forcefully thrown from a team.
 *
 * Copyright (c) 2001 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Lesser 
 * General Public License.
 * 
 * See the file "license-lesser.terms" for information on 
 * usage and redistribution of this file.
 */

package edu.utah.janosvm.sys;

/**
 * Exception thrown when a thread is forcefully thrown from a team
 * during a visit.  A thread can be forcefully ejected when it fails
 * to allocate memory and the VM cannot make progress without the memory.
 *
 * <p><b>NOTE</b>: This exception can be thrown by <em>any</em>
 * allocation, {@link Object#wait} or <tt>throw</tt> statement in the
 * code that executes during a visit, not just the <tt>switchTo</tt>
 * or <tt>returnFrom</tt> methods.
 *
 * <p><b>NOTE</b>: This exception is NOT CATCHABLE in the context of
 * the team from which the thread is being ejected.  The thread proceeds
 * immediately to the visit boundary, and then the exception is handled
 * normally from that point on.  For example:
 * <pre>
 * try
 * {
 *   team.{@link TeamHandle#switchTo switchTo}();
 *   try
 *   {
 *     VMInstance.crash(); // Force a TeamEjectionException
 *   }
 *   catch(TeamEjectionException e)
 *   {
 *     // <b>This won't run.</b>
 *   }
 *   finally
 *   {
 *     // <b>This won't run.</b>
 *   }
 *   team.{@link TeamHandle#returnFrom returnFrom}(); // Here is where control is returned.
 * }
 * catch(TeamEjectionException e)
 * {
 *   // <b>This will run</b>
 * }
 * </pre>
 *
 * <p>See {@link TeamReservation} for more details on visits and the
 * special handling of code running during a visit.
 *
 * @author Tim Stack, <a href="mailto:janos-discuss@flux.cs.utah.edu">Utah Janos Team</a>
 */
public final class TeamEjectionException
    extends Exception
{
    public TeamEjectionException()
    {
	super();
    }

    public TeamEjectionException(String msg)
    {
	super(msg);
    }
}
