/*
 * TeamHandle.java
 * A handle for accessing TeamBackEnd objects
 *
 * Copyright (c) 2000, 2001, 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Lesser 
 * General Public License.
 * 
 * See the file "license-lesser.terms" for information on 
 * usage and redistribution of this file.
 */

package edu.utah.janosvm.sys;

import java.util.Vector;
import edu.utah.janosvm.resources.ResourceSpec;
import edu.utah.janosvm.resources.ResourceSpecException;

/**
 * Provides a safe handle to a Team.  Technically, this points to the
 * kernel team's TeamBackEnd object that represents the team in
 * question.
 *
 * <p> Allocating a TeamHandle does not create a team.  See the {@link
 * Team#create create} methods in {@link Team}.
 *
 * @see TeamBackEnd
 * @see Team
 * @see EventTeamHandle
 * @author Tim Stack, <a href="mailto:janos-discuss@flux.cs.utah.edu">Utah Janos Team</a>
 */
public class TeamHandle
    extends Importable
    implements Cloneable
{
    protected TeamBackEnd be;

    public TeamHandle()
    {
    }
    
    /**
     * Copy constructor.
     */
    protected TeamHandle(TeamHandle other)
    {
	this.copy(other);
    }
    
    protected void bindTo(Exportable ex)
    {
	this.be = (TeamBackEnd)ex;
    }

    protected void unbindFrom()
    {
	this.be = null;
    }

    public boolean equals(Object obj)
    {
	boolean retval = false;
	
	if( obj instanceof TeamHandle )
	{
	    TeamHandle th = (TeamHandle)obj;

	    if( th.be == this.be )
		retval = true;
	}
	return retval;
    }
    
    /**
     * Get the name of the team associated with this
     * handle.
     *
     * @return A copy of the team's name.
     */
    public synchronized String getName()
    {
	String retval = null;

	if( this.be != null )
	{
	    String name;
	    
	    if( (name = this.be.getName()) != null )
		retval = CloneFactory.clone(name);
	    else
		retval = "(none)";
	}
	return retval;
    }

    /**
     * Get the Team's description string.
     *
     * @return A copy of the Team's description.
     */
    public synchronized String getDescription()
    {
	String retval = null;

	if( this.be != null )
	{
	    String desc;

	    if( (desc = this.be.getPeer().getDescription()) != null )
		retval = CloneFactory.clone(desc);
	}
	return retval;
    }

    /**
     * Returns the unique integer identifier for the team represented
     * by this handle.  Returns 0 if this handle is not connected, or
     * is connected to a dead team.
     *
     * @return The unique integer identifier for the team represented
     * by this handle.
     */
    public synchronized int getID()
    {
	int retval = 0;

	if( this.be != null )
	{
	    retval = this.be.getID();
	}
	return retval;
    }

    /**
     * @return The number of threads in the referenced team.
     */
    public synchronized int getThreadCount()
    {
	int retval = 0;

	if( this.be != null )
	{
	    retval = this.be.getThreadCount();
	}
	return retval;
    }

    /**
     * @return The number of daemon threads in the referenced team.
     */
    public synchronized int getDaemonCount()
    {
	int retval = 0;

	if( this.be != null )
	{
	    retval = this.be.getDaemonCount();
	}
	return retval;
    }

    /**
     * Test if the given object is "owned" by the team this handle
     * represents.  More specifically, test if it owns the memory
     * used to allocate the object.
     *
     * <p>Passing <tt>null</tt> returns true.
     *
     * @param obj The object to test for ownership.
     * @return True if this TeamHandle owns the object.  False if
     * not or if the handle has been revoked.
     */
    public synchronized boolean owns(Object obj)
    {
	if( this.be != null )
	    return this.be.owns(obj);
	else
	    return false;
    }

    /**
     * Terminate the team referenced by this handle.  
     * This will clean up all
     * the importables and exportables owned by the
     * team and schedule all of its threads for
     * destruction.
     * <p>
     * If the calling thread is owned by the team being
     * terminated, this function will not return.
     * <p>
     * This method blocks until after the {@link TeamEvent#TERMINATED} 
     * events are sent.  It may return before the {@link TeamEvent#BURIED}
     * events are sent.
     * Active visits to the team being terminated will
     * delay termination until all the visits have been completed.
     *
     * XXX if there is already a terminate in progress, I think
     * this will return early.
     * @see TeamBackEnd#terminate()
     */
    public synchronized void terminate()
    {
	if( this.be != null )
	{
	    this.be.terminate();
	    this.be = null;
	}
    }

    /**
     * Test if this handle has been revoked (which is frequently
     * associated with the death of the Team).  Does not actually tell
     * if the Team this handle was associated with is dead.
     *
     * <p>To reliably know if a Team is terminated, use an {@link
     * EventTeamHandle} to get a "terminated" or "buried" event.
     *
     * <p>XXX Perhaps this method should be moved to EventTeamHandle.
     */
    public synchronized boolean isTerminated()
    {
	boolean retval;

	if( this.be == null )
	    retval = true;
	else
	    retval = false;
	return retval;
    }
    
    public void importObject(Importable to, Object id)
	throws DeadTeamException,
	       TeamEjectionException,
	       NoSuchExportException
    {
	Team team = Team.current();

	team.getPortAuthority().importObject(to, this, id);
    }
    
    public void cancelImport(Importable im)
    {
	Team team = Team.current();

	team.getPortAuthority().cancelImport(im);
    }
    
    /**
     * Create a new reservation that reserves the team this
     * object points to.  The reservation remains "active" until
     * {@link TeamReservation#pop()} is invoked by the current thread.
     * Unlike a {@link TeamHandle#switchTo()}, this call does not
     * implicitly switch context to the other team.  That is
     * done by invoking {@link TeamReservation#visit()} on the
     * TeamReservation instance returned by this call.
     * 
     * <p>Unlike {@link #switchTo} when creating a reservation from
     * the current team to the current Team, termination of the
     * current Team <em>is</em> delayed until the reservation is
     * popped.
     *
     * <p>Never returns <tt>null</tt>.
     * 
     * <p>See {@link TeamReservation} for an example, and for details
     * of interaction with memory allocations, throwing exceptions, and
     * blocking in {@link Object#wait}.
     * 
     * @return A TeamReservation object that reserves this team.
     * @see #switchTo()
     * @see TeamBackEnd#pushReservation(int)
     * @see TeamReservation#visit()
     * @see TeamReservation#leave()
     */
    public synchronized TeamReservation pushReservation()
	throws DeadTeamException
    {
	TeamReservation retval = null;

	if( this.be != null )
	    retval = this.be.pushReservation(1);
	if( retval == null )
	    throw new DeadTeamException();
	return retval;
    }

    /**
     * Start a visit from the current team to the team this handle
     * refers to.  While the visit is active, neither team can be
     * terminated, so object references in either team can be
     * manipulated.  The visit ends when {@link #returnFrom()} is
     * invoked.
     *
     * <p>Note that switching to the current Team (i.e.,
     * <tt>Team.current().switchTo()</tt>) does NOT prevent the
     * current Team from being terminated.  Perhaps that is a bug.
     *
     * <p>See {@link TeamReservation} for an example, and for details
     * of interaction with memory allocations, throwing exceptions, and
     * blocking in {@link Object#wait}.
     *
     * @see TeamBackEnd#switchTo(int)
     */
    public synchronized void switchTo()
	throws DeadTeamException, TeamEjectionException
    {
	if( this.be == null )
	    throw new DeadTeamException();
	if( !this.be.switchTo(1) )
	    throw new DeadTeamException();
    }

    /**
     * End a visit to this team.
     *
     * <p>XXX Throws an exception/error if used without a corresponding switchTo?
     *
     * @see #switchTo()
     * @see TeamBackEnd#returnFrom()
     */
    public static void returnFrom()
    {
	TeamBackEnd.returnFrom();
    }
    
    /**
     * Pop the last TeamReservation off the stack.
     *
     * <p>Prefer use of {@link TeamReservation#pop()}.
     *
     * <b>NOTE:</b> the given tr is just used as a sanity check
     * to make sure an intervening reservation hasn't been skipped.
     */
    static void popReservation(TeamReservation tr)
    {
	TeamBackEnd.popReservation(tr);
    }

    public synchronized Object clone()
    {
	try
	{
	    TeamHandle retval = (TeamHandle)this.getClass().newInstance();
	    
	    retval.copy(this);
	    return retval;
	}
	catch(IllegalAccessException e)
	{
	    throw new InternalError();
	}
	catch(InstantiationException e)
	{
	    throw new InternalError();
	}
    }
    
    public String toString()
    {
	return "TeamHandle[name="
	    + this.getName()
	    + "; id="
	    + this.getID()
	    + "]";
    }
}
