/*
 * TeamLocal.java
 * A ThreadLocal like class that stores data local to a team.
 *
 * Copyright (c) 2002, 2003 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Lesser 
 * General Public License.
 * 
 * See the file "license-lesser.terms" for information on 
 * usage and redistribution of this file.
 */

package edu.utah.janosvm.sys;

import java.util.Hashtable;

/**
 * A {@link java.lang.ThreadLocal}-like class that stores data local
 * to a team.  Users should override {@link #initialValue} in a
 * subclass if they cannot control the order of {@link #get} and
 * {@link #set} invocations.  The default <code>initialValue</code>
 * returns <code>null</code>.
 *
 * @author Tim Stack, <a href="mailto:janos-discuss@flux.cs.utah.edu">Utah Janos Team</a>
 */
public class TeamLocal
{
    /**
     * Invoked at most once if a {@link #get} is invoked
     * before the first invocation of {@link #set}.
     *
     * @return The initial value of the TeamLocal.
     */
    protected Object initialValue()
    {
	return null;
    }

    /**
     * @return The current value of the TeamLocal.
     */
    public Object get()
    {
	return get(Team.current());
    }

    /**
     * @param value The value to put in the TeamLocal.
     */
    public void set(Object value)
    {
	set(Team.current(), value);
    }

    /**
     * Set a TeamLocal's value once.  Any future attempts to set the value will
     * fail and return false.
     *
     * @param value The value to put in the TeamLocal.
     * @return True if the TeamLocal was not previously set, false otherwise.
     */
    public boolean setOnce(Object value)
    {
	return setOnce(Team.current(), value);
    }
    
    /**
     * @param team The team to use as context when retrieving the local.
     * @return The current value of the TeamLocal.
     */
    private Object get(Team team)
    {
	Hashtable ht = team.getLocals();
	Object retval = null;
	
	synchronized( ht )
	{
	    if( (retval = ht.get(this)) == null )
	    {
		Object initialValue = this.initialValue();
		if (initialValue != null)
		    set(team, initialValue);
		retval = initialValue;
	    }
	}
	return retval;
    }

    /**
     * @param team The team to use as context when setting the local.
     * @param value The value to put in the local.
     */
    private void set(Team team, Object value)
    {
	Hashtable ht = team.getLocals();
	
	synchronized( ht )
	{
	    if( value != null )
	    {
		ht.put(this, value);
	    }
	    else
	    {
		ht.remove(this);
	    }
	}
    }

    /**
     * @param team The team to use as context when setting the local.
     * @param value The value to put in the local.
     * @return False if the TeamLocal was not previously set, true otherwise.
     */
    private boolean setOnce(Team team, Object value)
    {
	Hashtable ht = team.getLocals();
	boolean retval = true;
	
	synchronized( ht )
	{
	    if( ht.containsKey(this) )
	    {
		retval = false;
	    }
	    else
	    {
		ht.put(this, value);
	    }
	}
	return retval;
    }
    
    public String toString()
    {
	return this.getClass().getName() + "|Context[value="
	    + this.get()
	    + "]";
    }
}
